@@ -8,13 +8,26 @@ from typing import List, Tuple, Dict, Optional, Union
 
 LOG = logging.getLogger(__name__)
 
-class SettingPublisher(RedisClient):
+class SettingPublisher(object):
     #
-    def __init__(self, *args, **kwargs):
-        super(SettingPublisher, self).__init__(**kwargs)
-        self.CHANNEL_PREFIX = self.CHANNEL_GROUP + ':'
+    def __init__(self, *args, connector=None, **kwargs):
+        if isinstance(connector, RedisClient):
+            self.__use_shared_connector = True
+            self.__connector = connector
+        else:
+            self.__use_shared_connector = False
+            self.__connector = RedisClient(**kwargs)
+        #
+        self.CHANNEL_PREFIX = self.__connector.CHANNEL_GROUP + ':'
+        #
+        super(SettingPublisher, self).__init__()
     #
+    ##
+    @property
+    def connector(self):
+        return self.__connector
     #
+    ##
     def publish(self, message: Union[Dict, bytes, str, int, float],
             label: Optional[Union[bytes,str]] = None,
             with_datetime: Optional[bool] = False,
@@ -30,7 +43,7 @@ class SettingPublisher(RedisClient):
     #
     def __publish_or_error(self, message, label=None, with_datetime=False):
         if label is None:
-            channel_name = self.CHANNEL_GROUP
+            channel_name = self.__connector.CHANNEL_GROUP
         else:
             if isinstance(label, bytes):
                 label = label.decode('utf-8')
@@ -38,14 +51,14 @@ class SettingPublisher(RedisClient):
                 label = str(label)
             channel_name = self.CHANNEL_PREFIX + label
         #
-        if isinstance(message, dict):
+        if isinstance(message, (dict, list)):
             message, err = json_dumps(message, with_datetime=with_datetime)
             if err:
                 if LOG.isEnabledFor(logging.ERROR):
                     LOG.log(logging.ERROR, err)
                 raise err
         elif not self.__is_valid_type(message):
-            errmsg = "Invalid type of input: '%s'. Only a dict, bytes, string, int or float accepted." % type(message)
+            errmsg = "Invalid type of input: '%s'. Only a dict, list, bytes, string, int or float accepted." % type(message)
             if LOG.isEnabledFor(logging.ERROR):
                 LOG.log(logging.ERROR, errmsg)
             raise ValueError(errmsg)
@@ -53,11 +66,12 @@ class SettingPublisher(RedisClient):
         if LOG.isEnabledFor(logging.DEBUG):
             LOG.log(logging.DEBUG, "publish() a message [%s] to channel [%s]", str(message), channel_name)
         #
-        assure_not_null(self.connect(pinging=False, retrying=False)).publish(channel_name, message)
+        assure_not_null(self.__connector.connect(pinging=False, retrying=False)).publish(channel_name, message)
     #
     #
     def close(self):
-        super().close()
+        if not self.__use_shared_connector:
+            self.__connector.close()
         if LOG.isEnabledFor(logging.DEBUG):
             LOG.log(logging.DEBUG, "SettingPublisher has closed")
     #
