@@ -1,27 +1,39 @@
 #!/usr/bin/env python3
 
-import atexit
 import logging
-import redis, threading
+import redis
 import signal
+import threading
 import traceback, sys, os
 
 from configator.engine.connector import RedisClient
-from configator.utils.function import assure_not_null
+from configator.engine.container import SettingCapsule
+from configator.utils.function import assure_not_null, match_by_label, extract_parameters
 from typing import Any, Callable, List, Tuple, Dict, Optional
 
 LOG = logging.getLogger(__name__)
 
-class SettingSubscriber(RedisClient):
+class SettingSubscriber(object):
     #
-    def __init__(self, *args, **kwargs):
-        super(SettingSubscriber, self).__init__(**kwargs)
-        self.CHANNEL_PATTERN = self.CHANNEL_GROUP + '*'
+    def __init__(self, *args, connector=None, **kwargs):
+        if isinstance(connector, RedisClient):
+            self.__use_shared_connector = True
+            self.__connector = connector
+        else:
+            self.__use_shared_connector = False
+            self.__connector = RedisClient(**kwargs)
+        self.CHANNEL_PATTERN = self.__connector.CHANNEL_GROUP + '*'
+        super(SettingSubscriber, self).__init__()
+    #
+    ##
+    @property
+    def connector(self):
+        return self.__connector
     #
     ##
     @property
     def pubsub(self):
-        ps = assure_not_null(self.connect()).pubsub()
+        ps = assure_not_null(self.__connector.connect()).pubsub()
         ps.psubscribe(**{self.CHANNEL_PATTERN: self.__process_event})
         return ps
     #
@@ -44,38 +56,18 @@ class SettingSubscriber(RedisClient):
         with self.__pubsub_lock:
             if self.__pubsub_thread is not None:
                 self.__pubsub_thread.stop()
-            super().close()
+            #
+            if not self.__use_shared_connector:
+                self.__connector.close()
+            #
             if self.__pubsub_thread is not None:
                 self.__pubsub_thread.join()
                 self.__pubsub_thread = None
+            #
             if LOG.isEnabledFor(logging.DEBUG):
                 LOG.log(logging.DEBUG, "SettingSubscriber has stopped")
     #
     #
-    def hook_atexit(self):
-        def atexit_handler():
-            if LOG.isEnabledFor(logging.DEBUG):
-                LOG.log(logging.DEBUG, "AtExit occurred")
-            self.close()
-        atexit.register(atexit_handler)
-        return self
-    #
-    def hook_sigint(self, finished=True):
-        return self.hook_signal(signal_code=signal.SIGINT, finished=finished)
-    #
-    def hook_signal(self, signal_code=signal.SIGINT, finished=False):
-        current_handler = None
-        def signal_handler(signalnum, frame):
-            if LOG.isEnabledFor(logging.DEBUG):
-                LOG.log(logging.DEBUG, "SIGNAL[%d] received" % signalnum)
-            self.close()
-            if not finished and callable(current_handler):
-                if LOG.isEnabledFor(logging.DEBUG):
-                    LOG.log(logging.DEBUG, "Invoke the default handler")
-                current_handler(signalnum, frame)
-        current_handler = signal.signal(signal_code, signal_handler)
-        return self
-    #
     def __run_in_thread(self, auto_start=True, sleep_time=0, daemon=False):
         thread = PubSubWorkerThread(self, sleep_time, daemon=daemon)
         if auto_start:
@@ -117,14 +109,22 @@ class SettingSubscriber(RedisClient):
                 for reset in reaction:
                     if callable(reset):
                         reset(msg, err)
+    #
+    #
+    def register_receiver(self, capsule: SettingCapsule):
+        def wrap_capsule_reset(message, err):
+            return capsule.reset(parameters=extract_parameters(message, err))
+        if isinstance(capsule, SettingCapsule):
+            self.add_event_handler(match_by_label(capsule.name), wrap_capsule_reset)
+        return capsule
 
 
 class PubSubWorkerThread(threading.Thread):
-    def __init__(self, redis_client, sleep_time, daemon=False):
+    def __init__(self, subscriber, sleep_time, daemon=False):
         super(PubSubWorkerThread, self).__init__()
         self.daemon = daemon
         #
-        self._redis_client = redis_client
+        self._subscriber = subscriber
         self._sleep_time = sleep_time
         #
         self._running = threading.Event()
@@ -138,12 +138,12 @@ class PubSubWorkerThread(threading.Thread):
         self._running.set()
         while self._running.is_set():
             try:
-                pubsub = self._redis_client.pubsub
+                pubsub = self._subscriber.pubsub
                 while self._running.is_set():
                     pubsub.get_message(ignore_subscribe_messages=True, timeout=self._sleep_time)
                 pubsub.close()
             except redis.ConnectionError:
-                self._redis_client.reconnect()
+                self._subscriber.connector.reconnect()
             except Exception as err:
                 if LOG.isEnabledFor(logging.ERROR):
                     LOG.log(logging.ERROR, err)
